""" The onegov organisation homepage. """

from morepath import redirect
from onegov.core.security import Public
from onegov.directory import DirectoryCollection
from onegov.event import OccurrenceCollection
from onegov.form import FormCollection
from onegov.org import OrgApp
from onegov.org.homepage_widgets import inject_widget_variables
from onegov.org.layout import DefaultLayout
from onegov.org.models import Organisation
from onegov.org.models import PublicationCollection
from onegov.reservation import ResourceCollection


def redirect_to(request, target):
    if target == 'directories':
        return redirect(request.class_link(DirectoryCollection))

    if target == 'events':
        return redirect(request.class_link(OccurrenceCollection))

    if target == 'forms':
        return redirect(request.class_link(FormCollection))

    if target == 'publications':
        return redirect(request.class_link(PublicationCollection))

    if target == 'reservations':
        return redirect(request.class_link(ResourceCollection))


@OrgApp.html(model=Organisation, template='homepage.pt', permission=Public)
def view_org(self, request):
    """ Renders the org's homepage. """

    # the homepage can optionally be used as a jump-pad to redirect to
    # sub-part of the organisation -> this is useful if only one specific
    # module is used (e.g. only reservations)
    redirect = redirect_to(request, request.app.org.redirect_homepage_to)

    if redirect:
        return redirect

    layout = DefaultLayout(self, request)

    default = {
        'layout': layout,
        'title': self.title
    }

    structure = self.meta.get('homepage_structure')
    return inject_widget_variables(layout, structure, default)



from datetime import timedelta
from onegov.core.layout import Layout
from onegov.form import Form
from onegov.form.fields import MultiCheckboxField
from onegov.newsletter import Recipient
from onegov.org import _
from sedate import replace_timezone, to_timezone, utcnow
from wtforms import RadioField, StringField, TextAreaField, validators
from wtforms import ValidationError
from wtforms.fields.html5 import DateTimeField


class NewsletterForm(Form):

    title = StringField(
        label=_("Title"),
        description=_("Used in the overview and the e-mail subject"),
        validators=[validators.InputRequired()])

    lead = TextAreaField(
        label=_("Editorial"),
        description=_("A few words about this edition of the newsletter"),
        render_kw={'rows': 6})

    def update_model(self, model, request):
        model.title = self.title.data
        model.lead = self.lead.data
        model.html = self.get_html(request)

    def apply_model(self, model):
        self.title.data = model.title
        self.lead.data = model.lead

    def get_html(self, request):
        return ''

    @classmethod
    def with_news(cls, request, news):

        layout = Layout(None, request)

        choices = tuple((
            str(item.id),
            '<div class="title">{}</div><div class="date">{}</div>'.format(
                item.title,
                layout.format_date(item.created, 'relative')
            )
        ) for item in news)

        class NewsletterWithNewsForm(cls):

            news = MultiCheckboxField(
                label=_("Latest news"),
                choices=choices,
                render_kw={
                    'prefix_label': False,
                    'class_': 'recommended'
                },
                fieldset=_("Selected News / Events")
            )

            def update_model(self, model, request):
                super().update_model(model, request)
                model.content['news'] = self.news.data

            def apply_model(self, model):
                super().apply_model(model)
                self.news.data = model.content.get('news')

        return NewsletterWithNewsForm

    @classmethod
    def with_occurrences(cls, request, occurrences):

        choices = tuple(
            (
                str(item.id),
                '<div class="title">{}</div><div class="date">{}</div>'.format(
                    item.title, item.localized_start.strftime('%d.%m.%Y %H:%M')
                )
            ) for item in occurrences
        )

        class NewsletterWithOccurrencesForm(cls):

            occurrences = MultiCheckboxField(
                label=_("Events"),
                choices=choices,
                render_kw={
                    'prefix_label': False,
                    'class_': 'recommended'
                },
                fieldset=_("Selected News / Events")
            )

            def update_model(self, model, request):
                super().update_model(model, request)
                model.content['occurrences'] = self.occurrences.data

            def apply_model(self, model):
                super().apply_model(model)
                self.occurrences.data = model.content.get('occurrences')

        return NewsletterWithOccurrencesForm


class NewsletterSendForm(Form):

    send = RadioField(
        _("Send"),
        choices=(
            ('now', _("Now")),
            ('specify', _("At a specified time"))
        ),
        default='now'
    )

    time = DateTimeField(
        _("Time"),
        validators=[validators.InputRequired()],
        depends_on=('send', 'specify')
    )

    def validate_time(self, field):
        if not field.data:
            return

        from onegov.org.layout import DefaultLayout  # XXX circular import
        layout = DefaultLayout(self.model, self.request)

        if self.send.data == 'specify':
            time = replace_timezone(field.data, layout.timezone)
            time = to_timezone(time, 'UTC')

            if time < (utcnow() + timedelta(seconds=60 * 5)):
                raise ValidationError(_(
                    "Scheduled time must be at least 5 minutes in the future"
                ))

            if time.minute != 0:
                raise ValidationError(_(
                    "Newsletters can only be sent on the hour "
                    "(10:00, 11:00, etc.)"
                ))

            self.time.data = time


class NewsletterTestForm(Form):

    @classmethod
    def build(cls, newsletter, request):
        recipients = request.session.query(Recipient)\
            .with_entities(Recipient.id, Recipient.address)\
            .filter_by(confirmed=True)

        choices = tuple((r.id.hex, r.address) for r in recipients)

        class NewsletterSendFormWithRecipients(cls):

            selected_recipient = RadioField(
                label=_("Recipient"),
                choices=choices,
            )

            @property
            def recipient(self):
                return request.session.query(Recipient)\
                    .filter_by(id=self.selected_recipient.data)\
                    .one()

        return NewsletterSendFormWithRecipients



""" The settings view, defining things like the logo or color of the org. """

from onegov.core.security import Secret
from onegov.org.elements import Link
from onegov.org.layout import DefaultLayout
from onegov.org import _
from onegov.org.app import OrgApp
from onegov.org.forms import SettingsForm
from onegov.org.models import Organisation


@OrgApp.form(
    model=Organisation, name='settings', template='form.pt',
    permission=Secret, form=SettingsForm)
def handle_settings(self, request, form):
    """ Handles the GET and POST login requests. """

    layout = DefaultLayout(self, request)
    layout.include_editor()
    layout.include_code_editor()

    request.include('check_contrast')

    if form.submitted(request):
        form.populate_obj(self, exclude={'primary_color', 'footer-height'})
        self.theme_options = form.theme_options

        request.success(_("Your changes were saved"))

        @request.after
        def clear_locale(response):
            response.delete_cookie('locale')

        # always redirect to make sure locale changes are applied
        return request.redirect(request.link(self, name='settings'))

    elif request.method == 'GET':
        form.process(obj=self)
        form.theme_options = self.theme_options

    layout.breadcrumbs = [
        Link(_("Homepage"), layout.homepage_url),
        Link(_("Settings"), request.link(self))
    ]

    return {
        'layout': layout,
        'title': _('Settings'),
        'form': form,
        'form_width': 'huge'
    }



from collections import namedtuple
from onegov.directory import DirectoryCollection, DirectoryEntryCollection
from onegov.event import OccurrenceCollection
from onegov.newsletter import NewsletterCollection
from onegov.org import _, OrgApp
from onegov.org.elements import Link, LinkGroup
from onegov.org.layout import EventBaseLayout
from onegov.org.models import ImageSet, ImageFile
from onegov.file.models.fileset import file_to_set_associations
from sqlalchemy import func


@OrgApp.homepage_widget(tag='row')
class RowWidget(object):
    template = """
        <xsl:template match="row">
            <div class="row">
                <xsl:apply-templates select="node()"/>
            </div>
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='column')
class ColumnWidget(object):
    template = """
        <xsl:template match="column">
            <div class="small-12 medium-{@span} columns">
                <xsl:apply-templates select="node()"/>
            </div>
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='panel')
class PanelWidget(object):
    # panels with less than one link (not counting the more-link) are
    # hidden unless the user is logged-in
    template = """
        <xsl:template match="panel">
            <div class="side-panel requires-children">
                <xsl:attribute name="data-required-children">
                    <xsl:value-of select="'a:not(.more-link)'"/>
                </xsl:attribute>
                <xsl:attribute name="data-required-count">
                    <xsl:value-of select="'1'"/>
                </xsl:attribute>
                <xsl:attribute name="data-required-unless">
                    <xsl:value-of select="'.is-logged-in'"/>
                </xsl:attribute>

                <xsl:apply-templates select="node()"/>
            </div>
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='links')
class LinksWidget(object):
    template = """
        <xsl:template match="links">
            <xsl:if test="@title">
                <h2>
                    <xsl:value-of select="@title" />
                </h2>
            </xsl:if>
            <ul class="panel-links">
                <xsl:for-each select="link">
                <li>
                    <a>
                        <xsl:attribute name="href">
                            <xsl:value-of select="@url" />
                        </xsl:attribute>

                        <xsl:value-of select="node()" />
                    </a>

                    <xsl:if test="@description">
                        <small>
                            <xsl:value-of select="@description" />
                        </small>
                    </xsl:if>
                </li>
                </xsl:for-each>
            </ul>
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='directories')
class DirectoriesWidget(object):
    template = """
        <xsl:template match="directories">
            <metal:block use-macro="layout.macros['directories-panel']" />
        </xsl:template>
    """

    def get_variables(self, layout):
        directories = DirectoryCollection(
            layout.app.session(), type="extended")

        links = [
            Link(
                text=d.title,
                url=layout.request.class_link(
                    DirectoryEntryCollection, {'directory_name': d.name}
                ),
                subtitle=(
                    d.count == 1 and
                    _("1 entry") or
                    _("${count} entries", mapping={'count': d.count})
                )
            ) for d in layout.request.exclude_invisible(directories.query())
        ]

        links.append(
            Link(
                text=_("All directories"),
                url=layout.request.class_link(DirectoryCollection),
                classes=('more-link', )
            )
        )

        return {
            'directory_panel': LinkGroup(
                title=_("Directories"),
                links=links,
            )
        }


@OrgApp.homepage_widget(tag='news')
class NewsWidget(object):
    template = """
        <xsl:template match="news">
            <div metal:use-macro="layout.macros.newslist"
                tal:define="heading 'h3'; show_all_news_link True;
                hide_date True"
            />
        </xsl:template>
    """

    def get_variables(self, layout):
        # request more than the required amount of news to account for hidden
        # items which might be in front of the queue
        news = layout.request.exclude_invisible(
            layout.root_pages[-1].news_query(limit=4).all())

        # limits the news, but doesn't count sticky news towards that limit
        def limited(news, limit):
            count = 0

            for item in news:
                if count < limit or item.is_visible_on_homepage:
                    yield item

                if not item.is_visible_on_homepage:
                    count += 1

        return {
            'news': limited(news, limit=2)
        }


@OrgApp.homepage_widget(tag='homepage-cover')
class CoverWidget(object):
    template = """
        <xsl:template match="homepage-cover">
            <div class="homepage-content page-text">
                <tal:block
                    content="structure layout.org.meta.get('homepage_cover')"
                />
            </div>
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='events')
class EventsWidget(object):
    template = """
        <xsl:template match="events">
            <metal:block use-macro="layout.macros['events-panel']" />
        </xsl:template>
    """

    def get_variables(self, layout):
        occurrences = OccurrenceCollection(layout.app.session()).query()
        occurrences = occurrences.limit(4)

        event_layout = EventBaseLayout(layout.model, layout.request)
        event_links = [
            Link(
                text=o.title,
                url=layout.request.link(o),
                subtitle=event_layout.format_date(o.localized_start, 'event')
                .title()
            ) for o in occurrences
        ]

        event_links.append(
            Link(
                text=_("All events"),
                url=event_layout.events_url,
                classes=('more-link', )
            )
        )

        latest_events = LinkGroup(
            title=_("Events"),
            links=event_links,
        )

        return {
            'event_panel': latest_events
        }


@OrgApp.homepage_widget(tag='homepage-tiles')
class TilesWidget(object):
    template = """
        <xsl:template match="homepage-tiles">
            <xsl:choose>
                <xsl:when test="@show-title">
                    <metal:block use-macro="layout.macros['homepage-tiles']"
                       tal:define="show_title True" />
                </xsl:when>
                <xsl:otherwise>
                    <metal:block use-macro="layout.macros['homepage-tiles']"
                      tal:define="show_title False" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:template>
    """

    def get_variables(self, layout):
        return {'tiles': tuple(self.get_tiles(layout))}

    def get_tiles(self, layout):
        Tile = namedtuple('Tile', ['page', 'links', 'number'])

        homepage_pages = layout.request.app.homepage_pages
        request = layout.request
        link = request.link
        session = layout.app.session()
        classes = ('tile-sub-link', )

        for ix, page in enumerate(layout.root_pages):
            if page.type == 'topic':

                children = homepage_pages.get(page.id, tuple())
                children = (session.merge(c, load=False) for c in children)

                if not request.is_manager:
                    children = (
                        c for c in children if not c.is_hidden_from_public
                    )

                yield Tile(
                    page=Link(page.title, link(page)),
                    number=ix + 1,
                    links=tuple(
                        Link(c.title, link(c), classes=classes, model=c)
                        for c in children
                    )
                )
            elif page.type == 'news':
                news_url = link(page)
                years = (str(year) for year in page.years)

                links = [
                    Link(year, news_url + '?year=' + year, classes=classes)
                    for year in years
                ]

                links.append(Link(
                    _("Newsletter"), link(NewsletterCollection(session)),
                    classes=classes
                ))

                yield Tile(
                    page=Link(page.title, news_url),
                    number=ix + 1,
                    links=links
                )
            else:
                raise NotImplementedError


@OrgApp.homepage_widget(tag='line')
class HrWidget(object):
    template = """
        <xsl:template match="line">
            <hr />
        </xsl:template>
    """


@OrgApp.homepage_widget(tag='slider')
class SliderWidget(object):
    template = """
        <xsl:template match="slider">
            <div metal:use-macro="layout.macros.slider" />
        </xsl:template>
    """

    def get_images_from_sets(self, layout):
        session = layout.app.session()

        sets = session.query(ImageSet)
        sets = sets.with_entities(ImageSet.id, ImageSet.meta)
        sets = tuple(
            s.id for s in sets
            if s.meta.get('show_images_on_homepage') and not
            s.meta.get('is_hidden_from_public')
        )

        if not sets:
            return

        files = session.query(file_to_set_associations)
        files = files.with_entities(file_to_set_associations.c.file_id)
        files = files.filter(file_to_set_associations.c.fileset_id.in_(
            sets
        ))

        images = session.query(ImageFile)
        images = images.filter(ImageFile.id.in_(files.subquery()))
        images = images.order_by(func.random())
        images = images.limit(6)

        for image in images:
            yield {
                'note': image.note,
                'src': layout.request.link(image)
            }

    def get_images_from_theme(self, layout):
        for key, value in layout.org.theme_options.items():
            if key.startswith('tile-image'):
                yield {
                    'note': None,
                    'src': value.strip('"\'')
                }

    def get_variables(self, layout):
        # if we don't have an album used for images, we use the images
        # shown on the homepage anyway to avoid having to show nothing
        images = tuple(self.get_images_from_sets(layout)) \
            or tuple(self.get_images_from_theme(layout))

        return {
            'images': images
        }



from onegov.core.security import Private
from onegov.org import OrgApp
from onegov.org.models import PersonMove


@OrgApp.view(model=PersonMove, permission=Private, request_method='PUT')
def move_page(self, request):
    request.assert_valid_csrf_token()
    self.execute()



import pytest
import requests
import transaction

from datetime import datetime
from onegov.core.utils import module_path
from onegov.directory import DirectoryCollection
from onegov.file import FileCollection
from onegov_testing.utils import create_image
from pytz import UTC
from sedate import utcnow
from tempfile import NamedTemporaryFile
from time import sleep


def test_browse_activities(browser):
    # admins
    browser.login_admin()
    browser.visit('/timeline')

    assert browser.is_text_present("Noch keine Aktivität")

    # anonymous
    other = browser.clone()
    other.visit('/timeline')

    assert not other.is_text_present("Noch keine Aktivität")


@pytest.mark.parametrize("field", (
    'Photo = *.png|*.jpg',
    'Photo *= *.png|*.jpg',
))
def test_browse_directory_uploads(browser, org_app, field):
    DirectoryCollection(org_app.session(), type='extended').add(
        title="Crime Scenes",
        structure="""
            Name *= ___
            Description *= ...
            {field}
        """.format(field=field),
        configuration="""
            title:
                - name
            order:
                - name
            display:
                content:
                    - name
                    - description
                    - photo
        """,
        type='extended'
    )

    transaction.commit()

    # create a new page with a picture
    photo = create_image(output=NamedTemporaryFile(suffix='.png'))

    browser.login_admin()
    browser.visit('/directories/crime-scenes/+new')
    browser.fill('name', "Seven Seas Motel")
    browser.fill('description', "First victim of Ice Truck Killer")
    browser.fill('photo', photo.name)
    browser.find_by_value("Absenden").click()

    assert browser.is_text_present("Seven Seas Motel")
    assert browser.is_element_present_by_css('.field-display img')

    src = browser.find_by_css('.field-display img')['src']

    # elect to keep the picture (default)
    browser.find_by_css('.edit-link').click()
    browser.fill('name', "Seven Seas Motel, Miami")
    browser.find_by_value("Absenden").click()

    assert browser.is_text_present("Seven Seas Motel, Miami")
    assert browser.is_element_present_by_css('.field-display img')
    assert browser.find_by_css('.field-display img')['src'] == src

    # elect to replace the picture
    photo = create_image(output=NamedTemporaryFile(suffix='.png'))

    browser.find_by_css('.edit-link').click()
    browser.choose('photo', 'replace')
    browser.find_by_name('photo').last.value = photo.name
    browser.find_by_value("Absenden").click()

    assert browser.is_element_present_by_css('.field-display img')
    assert browser.find_by_css('.field-display img')['src'] != src

    # elect to delete the picture
    browser.find_by_css('.edit-link').click()
    browser.choose('photo', 'delete')
    browser.find_by_value("Absenden").click()

    if field.startswith('Photo ='):
        assert not browser.is_element_present_by_css('.field-display img')
    else:
        assert browser.is_text_present("Dieses Feld wird benötigt")


def test_upload_image_with_error(browser, org_app):
    DirectoryCollection(org_app.session(), type='extended').add(
        title="Crime Scenes",
        structure="""
            Name *= ___
            Description *= ...
            Photo = *.jpg|*.png
        """,
        configuration="""
            title:
                - name
            order:
                - name
            display:
                content:
                    - name
                    - description
                    - photo
        """,
        type='extended'
    )

    transaction.commit()

    # create a new page with a missing field (but do supply the picture)
    photo = create_image(output=NamedTemporaryFile(suffix='.png'))

    browser.login_admin()
    browser.visit('/directories/crime-scenes/+new')
    browser.fill('name', "Seven Seas Motel")
    browser.fill('photo', photo.name)
    browser.find_by_value("Absenden").click()

    assert browser.is_text_present("Dieses Feld wird benötigt")

    # try again with the missing field present
    browser.fill('description', "First victim of Ice Truck Killer")
    browser.find_by_value("Absenden").click()

    assert browser.is_text_present("Seven Seas Motel")

    # the image won't be there however (it gets cleared in between requests)
    assert not browser.is_element_present_by_css('.field-display img')


def test_browse_directory_editor(browser, org_app):
    browser.login_admin()
    browser.visit('/directories/+new')

    assert browser.is_element_present_by_css('.formcode-toolbar', wait_time=5)

    browser.fill('title', "Restaurants")

    # add a title through the dropdown menu
    browser.find_by_css('.formcode-toolbar-element').click()
    browser.find_by_css('.formcode-snippet-name')[0].click()

    # add a text through the dropdown menu
    browser.find_by_css('.formcode-toolbar-element').click()
    browser.find_by_css('.formcode-snippet-name')[1].mouse_over()
    browser.find_by_css('.formcode-snippet-required')[0].click()

    assert browser.find_by_css('#structure').value == (
        "# Titel\n"
        "Text *= ___\n"
    )

    # Add the title to the title format
    browser.scroll_to_css('#title_format')
    assert browser.is_element_present_by_xpath(
        '(//div[@class="formcode-toolbar-element"])[2]', wait_time=5)

    browser.find_by_css('.formcode-toolbar-element')[1].click()
    browser.find_by_css('.formcode-snippet-name')[0].click()

    assert browser.find_by_css('#title_format').value == '[Titel/Text]'

    # Add it to the lead format as well
    browser.scroll_to_css('#lead_format')
    assert browser.is_element_present_by_xpath(
        '(//div[@class="formcode-toolbar-element"])[3]', wait_time=5)

    browser.find_by_css('.formcode-toolbar-element')[2].click()
    browser.find_by_css('.formcode-snippet-name')[0].click()

    assert browser.find_by_css('#lead_format').value == '[Titel/Text]'

    # Elect to show the fields in the main view
    browser.find_by_css('.formcode-select label').click()

    assert browser.find_by_css('#content_fields').value == "Titel/Text"

    # Save the form and ensure that after the load we get the same selections
    browser.find_by_value("Absenden").click()
    browser.find_by_css('.edit-link').click()

    assert browser.find_by_css('.formcode-select input')[0].checked
    assert not browser.find_by_css('.formcode-select input')[1].checked


def test_browse_directory_coordinates(browser, org_app):
    DirectoryCollection(org_app.session(), type='extended').add(
        title="Restaurants",
        structure="""
            Name *= ___
            Tables = 0..1000
        """,
        configuration="""
            title: '[name]'
            lead: 'Offers [tables] tables'
            order:
                - name
            display:
                content:
                    - name
        """,
        meta={
            'enable_map': 'everywhere'
        },
        type='extended'
    )

    transaction.commit()

    # create two restaurants with two different coordinates
    browser.login_admin()
    browser.visit('/directories/restaurants/+new')
    browser.fill('name', "City Wok")
    browser.fill('tables', "10")
    assert browser.is_element_present_by_css('.add-point-active', wait_time=5)
    browser.execute_script('document.leafletmaps[0].panBy([-100, 100]);')
    browser.find_by_css('.add-point-active').click()
    browser.find_by_value("Absenden").click()

    browser.visit('/directories/restaurants/+new')
    browser.fill('name', "City Sushi")
    browser.fill('tables', "20")
    assert browser.is_element_present_by_css('.add-point-active', wait_time=5)
    browser.execute_script('document.leafletmaps[0].panBy([100, -100]);')
    browser.find_by_css('.add-point-active').click()
    browser.find_by_value("Absenden").click()

    # make sure the restaurants are visible in the overview
    browser.visit('/directories/restaurants')
    assert "Offers 20 tables" in browser.html
    assert "Offers 10 tables" in browser.html

    # as well as their points, which we can toggle
    assert not browser.is_element_present_by_css('.popup-title')
    assert not browser.is_element_present_by_css('.popup-lead')

    browser.find_by_css('.vector-marker')[1].click()
    assert browser.is_element_present_by_css('.popup-title')
    assert browser.is_element_present_by_css('.popup-lead')

    # the popup leads us to the restaurant
    browser.find_by_css('.popup-title').click()
    assert browser.is_element_present_by_id(
        'page-directories-restaurants-city-wok')


def test_publication_workflow(browser, temporary_path, org_app):
    path = temporary_path / 'foo.txt'

    with path.open('w') as f:
        f.write('bar')

    browser.login_admin()
    browser.visit('/files')

    # upload a file
    assert not browser.is_text_present("Soeben hochgeladen")
    browser.drop_file('.upload-dropzone', path)
    assert browser.is_text_present("Soeben hochgeladen")

    # show the details
    browser.find_by_css('.upload-filelist .untoggled').click()
    assert browser.is_text_present("Öffentlich")
    assert not browser.is_text_present("Privat")
    assert not browser.is_text_present("Publikationsdatum")

    # make sure the file can be downloaded
    file_url = browser.find_by_css('.file-preview')['href']
    r = requests.get(file_url)
    assert r.status_code == 200
    assert 'public' in r.headers['cache-control']

    # make sure unpublishing works
    browser.find_by_css('.publication .file-status-tag a').click()
    r = requests.get(file_url)
    assert r.status_code == 403

    assert browser.is_text_present("Privat", wait_time=1)
    assert browser.is_text_present("Publikationsdatum")
    assert not browser.is_text_present("Öffentlich")

    # enter a publication date in the past (no type date support in selenium)
    browser.find_by_css('select').select('07:00')

    assert browser.is_text_present("Wird publiziert am", wait_time=1)
    assert not browser.is_text_present("Publikationsdatum")

    dt = datetime.today()
    f = FileCollection(org_app.session()).query().one()

    assert f.publish_date in (
        datetime(dt.year, dt.month, dt.day, 5, tzinfo=UTC),  # dst
        datetime(dt.year, dt.month, dt.day, 6, tzinfo=UTC)   # !dst
    )

    # run the cronjob and make sure it works
    job = org_app.config.cronjob_registry.cronjobs['hourly_maintenance_tasks']
    job.app = org_app
    job_url = f'{browser.url.replace("/files", "")}/cronjobs/{job.id}'

    requests.get(job_url)
    sleep(0.1)

    r = requests.get(file_url)
    assert r.status_code == 200
    assert 'public' in r.headers['cache-control']


def test_signature_workflow(browser, temporary_path, org_app):
    path = module_path('onegov.org', 'tests/fixtures/sample.pdf')
    org_app.enable_yubikey = True

    # upload the pdf
    browser.login_admin()
    browser.visit('/files')
    browser.drop_file('.upload-dropzone', path)

    assert browser.is_text_present("Soeben hochgeladen")

    # show the details
    browser.find_by_css('.upload-filelist .untoggled').click()
    assert browser.is_text_present("Unsigniert")

    # try to sign the pdf (this won't work in this test-environment due to
    # it being in a different process, but we should see the error handling)
    browser.find_by_css('a.is-not-signed').click()

    assert browser.is_text_present("Bitte geben Sie Ihren Yubikey ein")
    assert browser.is_text_present("Signieren")

    browser.find_by_css('.dialog input').fill('foobar')
    browser.find_by_text("Signieren").click()

    assert browser.is_text_present("nicht mit einem Yubikey verknüpft")

    # change the database and show the information in the browser
    f = FileCollection(org_app.session()).query().one()
    f.signed = True
    f.signature_metadata = {
        'signee': 'foo@example.org',
        'timestamp': utcnow().isoformat()
    }
    transaction.commit()

    # make sure the signature information is shown
    browser.visit('/files')
    browser.find_by_css('.untoggled').click()

    assert browser.is_text_present('foo@example.org')
    assert browser.is_text_present('Digital signiert')



import json

from cached_property import cached_property
from datetime import date, datetime, timedelta
from dateutil import rrule
from dateutil.rrule import rrulestr
from onegov.form import Form
from onegov.form.fields import MultiCheckboxField
from onegov.gis import CoordinatesField
from onegov.org import _
from sedate import replace_timezone, to_timezone
from wtforms import RadioField, StringField, TextAreaField, validators
from wtforms.fields.html5 import DateField, EmailField
from wtforms_components import TimeField


TAGS = [(tag, tag) for tag in (
    _("Exhibition"),
    _("Gastronomy"),
    _("Health"),
    _("Cinema"),
    _("Congress"),
    _("Concert"),
    _("Art"),
    _("Literature"),
    _("Market"),
    _("Music"),
    _("Party"),
    _("Politics"),
    _("Religion"),
    _("Sports"),
    _("Dancing"),
    _("Theater"),
    _("Meetup"),
    _("Talk"),
)]

WEEKDAYS = (
    ("MO", _("Mo")),
    ("TU", _("Tu")),
    ("WE", _("We")),
    ("TH", _("Th")),
    ("FR", _("Fr")),
    ("SA", _("Sa")),
    ("SU", _("Su")),
)


class EventForm(Form):
    """ Defines the form for all events. """

    timezone = 'Europe/Zurich'

    email = EmailField(
        label=_("E-Mail"),
        description="max.muster@example.org",
        validators=[validators.InputRequired(), validators.Email()]
    )

    title = StringField(
        label=_("Title"),
        description=_("Concerto in the castle garden"),
        validators=[validators.InputRequired()]
    )

    description = TextAreaField(
        label=_("Description"),
        description=_("Enjoy a concerto in the castle garden."),
        render_kw={'rows': 12}
    )

    location = StringField(
        label=_("Location"),
        description=_("Castle garden"),
        validators=[validators.InputRequired()]
    )

    organizer = StringField(
        label=_("Organizer"),
        description=_("Music society"),
        validators=[validators.InputRequired()]
    )

    coordinates = CoordinatesField(
        label=_("Coordinates"),
        description=_("The marker can be moved by dragging it with the mouse"),
        render_kw={'data-map-type': 'marker'}
    )

    tags = MultiCheckboxField(
        label=_("Tags"),
        choices=TAGS,
    )

    start_date = DateField(
        label=_("Date"),
        validators=[validators.InputRequired()],
        default=date.today
    )

    start_time = TimeField(
        label=_("From"),
        description="18:00",
        validators=[validators.InputRequired()]
    )

    end_time = TimeField(
        label=_("To"),
        description="19:15",
        validators=[validators.InputRequired()]
    )

    repeat = RadioField(
        label=_("Repeat"),
        default='none',
        choices=(
            ('without', _("Without")),
            ('weekly', _("Weekly")),
            ('dates', _("On additional dates"))
        )
    )

    weekly = MultiCheckboxField(
        label=_("Repeats itself every"),
        choices=WEEKDAYS,
        render_kw={'prefix_label': False, 'class_': 'oneline-checkboxes'},
        depends_on=('repeat', 'weekly')
    )

    end_date = DateField(
        label=_("Until date"),
        validators=[validators.Optional()],
        depends_on=('repeat', 'weekly')
    )

    dates = TextAreaField(
        label=_("Dates"),
        depends_on=('repeat', 'dates'),
        render_kw={'class_': 'many many-dates'},
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.date_errors = {}

    def on_request(self):
        self.request.include('common')
        self.request.include('many')

        if not self.dates.data:
            self.dates.data = self.dates_to_json(None)

    def validate(self):
        """ Make sure a valid RRULE can be generated with the given fields.

        Might be better to group weekly and end_date in an enclosure,
        see See http://wtforms.readthedocs.org/en/latest/fields.html
        #field-enclosures.

        """
        result = super().validate()

        if self.start_date.data and self.end_date.data:
            if self.start_date.data > self.end_date.data:
                message = _("The end date must be later than the start date.")
                self.end_date.errors.append(message)
                result = False

        if self.repeat.data == 'weekly':
            if self.weekly.data and self.start_date.data:
                weekday = WEEKDAYS[self.start_date.data.weekday()][0]
                if weekday not in self.weekly.data:
                    message = _(
                        "The weekday of the start date must be selected.")
                    self.weekly.errors.append(message)
                    result = False

            if self.weekly.data and not self.end_date.data:
                message = _(
                    "Please set and end date if the event is recurring.")
                self.end_date.errors.append(message)
                result = False

            if self.end_date.data and not self.weekly.data:
                message = _(
                    "Please select a weekday if the event is recurring.")
                self.weekly.errors.append(message)
                result = False

        return result

    def populate_obj(self, model):
        """ Stores the form values on the model. """

        super().populate_obj(model, exclude={
            'start_date',
            'start_time',
            'end_date',
            'end_time',
            'weekly',
            'email',
        })

        # clear the recurrence to avoid updating all occurrences too much
        model.recurrence = ''

        model.meta = model.meta or {}
        model.meta['submitter_email'] = self.email.data
        model.timezone = 'Europe/Zurich'

        model.start = replace_timezone(
            datetime(
                self.start_date.data.year,
                self.start_date.data.month,
                self.start_date.data.day,
                self.start_time.data.hour,
                self.start_time.data.minute
            ),
            model.timezone
        )
        end_date = self.start_date.data
        if self.end_time.data <= self.start_time.data:
            end_date += timedelta(days=1)

        model.end = replace_timezone(
            datetime(
                end_date.year,
                end_date.month,
                end_date.day,
                self.end_time.data.hour,
                self.end_time.data.minute
            ),
            model.timezone
        )

        if self.repeat.data == 'without':
            self.recurrence = None
        elif self.repeat.data == 'weekly':
            if self.weekly.data and self.end_date.data:
                until_date = to_timezone(
                    replace_timezone(
                        datetime(
                            self.end_date.data.year,
                            self.end_date.data.month,
                            self.end_date.data.day,
                            self.end_time.data.hour,
                            self.end_time.data.minute
                        ), model.timezone
                    ), 'UTC'
                )
                model.recurrence = (
                    "RRULE:FREQ=WEEKLY;WKST=MO;BYDAY={0};UNTIL={1}".format(
                        ','.join(self.weekly.data),
                        until_date.strftime('%Y%m%dT%H%M%SZ')
                    )
                )
        elif self.repeat.data == 'dates':
            model.recurrence = '\n'.join(
                f"RDATE:{d.strftime('%Y%m%dT000000Z')}"
                for d in self.parsed_dates
            )
        else:
            raise NotImplementedError

    def process_obj(self, model):
        """ Stores the page values on the form. """

        super().process_obj(model)

        self.start_time.data = model.localized_start.time()
        self.end_time.data = model.localized_end.time()
        self.start_date.data = model.localized_start.date()

        if model.recurrence:
            last_occurrence = model.occurrence_dates(localize=True)[-1]
            self.end_date.data = last_occurrence.date()

            rule = rrulestr(model.recurrence)

            if getattr(rule, '_freq', None) == rrule.WEEKLY:
                self.repeat.data = 'weekly'
                self.dates.data = self.dates_to_json(None)
                self.weekly.data = [
                    WEEKDAYS[day][0] for day in rule._byweekday
                ]
            else:
                self.repeat.data = 'dates'
                self.dates.data = self.dates_to_json(tuple(rule))
        else:
            self.dates.data = self.dates_to_json(None)
            self.repeat.data = 'without'

        if model.meta:
            self.email.data = model.meta.get('submitter_email')

    @cached_property
    def parsed_dates(self):
        return self.json_to_dates(self.dates.data)

    def json_to_dates(self, text=None):
        result = []

        for value in json.loads(text or '{}').get('values', []):
            result.append(date(*map(int, value['date'].split('-'))))

        return result

    def dates_to_json(self, dates=None):
        dates = dates or []

        return json.dumps({
            'labels': {
                'date': self.request.translate(_("Date")),
                'add': self.request.translate(_("Add")),
                'remove': self.request.translate(_("Remove")),
            },
            'values': [
                {
                    'date': d.strftime('%Y-%m-%d'),
                    'error': self.date_errors.get(ix, "")
                } for ix, d in enumerate(dates)
            ]
        })



import pytest

from lxml import etree
from onegov.core.utils import scan_morepath_modules, Bunch
from onegov.org import OrgApp
from onegov.org.homepage_widgets.core import (
    parse_structure,
    inject_widget_variables,
    transform_homepage_structure
)
from wtforms import ValidationError


@pytest.mark.parametrize('invalid_structure', [
    " <panel><?python assert False></panel>",
    "<panel>${{request.password}}</panel>",
    "<panel>${request.password}</panel>",
    "<panel tal:content='request.password'></panel>",
    "<div>html</div>"

])
def test_parse_invalid_structure(invalid_structure):

    class App(OrgApp):
        pass

    scan_morepath_modules(App)
    App.commit()

    widgets = App().config.homepage_widget_registry.values()

    with pytest.raises(ValidationError):
        parse_structure(widgets, invalid_structure)


def test_inject_widget_variables():

    class App(OrgApp):
        pass

    @App.homepage_widget(tag='foo')
    class FooWidget(object):

        def get_variables(self, layout):
            return {
                'bar': '0xdeadbeef'
            }

    scan_morepath_modules(App)
    App.commit()

    layout = Bunch(app=App())
    assert inject_widget_variables(layout, "") is None
    assert inject_widget_variables(layout, "<foo />") == {'bar': '0xdeadbeef'}
    assert inject_widget_variables(layout, "<foo />", {'foo': 'bar'}) == {
        'foo': 'bar',
        'bar': '0xdeadbeef'
    }
    assert inject_widget_variables(layout, "<panel />") == {}


def test_transform_homepage_structure():

    class App(OrgApp):
        pass

    scan_morepath_modules(App)
    App.commit()

    result = transform_homepage_structure(App(), """
        <row>
            <column span="8">
                <links>
                    <link url="#about" description="About">
                        Seantis
                    </link>
                </links>
            </column>
        </row>
    """)

    xml = etree.fromstring(result.encode('utf-8'))

    for ns in ('i18n', 'metal', 'tal'):
        assert ns in xml.nsmap

    assert xml.attrib['class'] == 'homepage'

    row = next(xml.iterchildren())
    assert row.tag == 'div'
    assert row.attrib['class'] == 'row'

    col = next(row.iterchildren())
    assert col.tag == 'div'
    assert col.attrib['class'] == 'small-12 medium-8 columns'

    ul = next(col.iterchildren())
    assert ul.tag == 'ul'
    assert ul.attrib['class'] == 'panel-links'

    li = next(ul.iterchildren())
    assert li.tag == 'li'

    a, small = li.getchildren()
    assert a.tag == 'a'
    assert a.attrib['href'] == '#about'
    assert 'Seantis' in a.text

    assert small.tag == 'small'
    assert 'About' in small.text
